#include "Mlista.h"

namespace MLista {

  //FUNCIONES AUXILIARES
  void insertarOrdenadoPorSimbolo(TLista &l, TElemento elem, TError &error);
  
  /* Devuelve cierto si la lista est vaca*/
  bool listaVacia(TLista l)
    {
       return (l==NULL);           
    }
 
   /* crea una lista vaca */
  void crearLista(TLista &l)
  {
       l = NULL;
   }  

  /* Destruye una lista previamente creada */
  void destruirLista(TLista &l)
  {
      TLista aux;
       
      while (l!=NULL)
      {
             aux = l;
             l = l->sig;
             delete(aux);
      } 
   }
  
  /* Inserta en la lista un elemento ordenado por su nmero. 
     Si el elemento ya existe se produce un error. */
  void insertarLista(TLista &l, TElemento elem, TError &error)
  {
       TLista nuevo;
         
       if ( (listaVacia(l)) || (elem.numero < l->elem.numero) )
         {   
                error = NoError;
                nuevo = new (TNodo);
                nuevo->elem = elem;
                nuevo->sig = l;
                l = nuevo;
             
         }
         else if (elem.numero==l->elem.numero)
         {  
            error = ErrorElementoYaExiste;
         }
         else
         {   
            insertarLista(l->sig,elem,error);
         } 
  }
  /* Elimina de la lista un elemento dado su nmero. 
     Si el elemento no existe se produce un error.*/
  void eliminarLista(TLista &l, TNatural num, TError &error)
  {
        TLista ptr;
         
         if ( (listaVacia(l)) || (num < l->elem.numero) )
         {
             error = ErrorNoEncontrado;
         }
         else if (num==l->elem.numero)
         {
             error = NoError;
             ptr = l;
             l = l->sig;
             delete(ptr);
         }
         else
         {
             eliminarLista(l->sig,num,error);
         }
  }

 
  /* Devuelve, si existe, un elemento dado su nmero. En caso de que no exista se producira un error. */
  void buscarLista(TLista l, TNatural num, TElemento &elem, TError &error)
  {
 
         if (listaVacia(l) || (num < l->elem.numero) ) 
         {
             error = ErrorNoEncontrado;
         }
         else if (num == l->elem.numero)
         {
              
             error = NoError;
             elem = l->elem;
         }
         else
         {
             buscarLista(l->sig,num, elem, error);
         }
  } 
  
     
  /* Imprime por pantalla la lista enlazada */  
  /*
    GRUPO 1:
    Hidrogeno 1 1 H 1 1
    Potasio 4 1 K 19 39.09
    Cesio 6 1 Cs 55 132.90
    Francio 7 1 Fr 87 223.01
  */            
  void escribirLista(TLista l)
  {
      TLista ptr;
        
      if (!listaVacia(l))
      {
         ptr = l;
         cout << "GRUPO: " << l->elem.grupo << endl;
         while (ptr!=NULL)
         {
           escribirElemento(ptr->elem);
           ptr = ptr->sig;
         }
         cout << endl;
      }
  } 
  
 /* Muestra por pantalla los smbolos de una lista de elementos ordenados por orden alfabtico.*/
  void ordenarLista (TLista &lista)
  {
       TLista temp, aux;
       TError error;
       
       crearLista(temp);
       aux=lista;
       while (aux!=NULL)
       {
             insertarOrdenadoPorSimbolo(temp, aux->elem, error);
             aux = aux ->sig;
       }
       destruirLista(lista);
       lista = temp;
       escribirLista(lista);
   }
  
  /* Dada una lista de elementos muestra por pantalla aquellos cuya masa atmica est entre min y max (ambos incluidos).*/
  void filtrarLista (TLista lista, double min, double max)
  {
       TLista aux;
       if (!listaVacia(lista)) 
       {
          aux=lista;
          cout << "GRUPO: " << lista->elem.grupo << endl;    
          while (aux!=NULL)
          {
             if ((aux->elem.masa >=min) && (aux->elem.masa<=max)) 
             {
                escribirElemento(aux->elem);
             }
             aux = aux ->sig;
          }        
         cout << endl;
       }
   }
  
  /* Vuelca al fichero con nombre nombre_fichero el contenido de una tabla.*/
  void escribirFicheroBIN (ofstream &fich, TLista lista, TError error)
  {
      TLista ptr;
        
      if (!listaVacia(lista))
      {
         ptr = lista;
         while (ptr!=NULL)
         {
           escribirElementoFicheroBIN(fich,ptr->elem);
           ptr = ptr->sig;
         }
      }
   }
   
   
  //FUNCIONES AUXILIARES
  
  /* Inserta en la lista un elemento ordenado por su simbolo. 
     Si el elemento ya existe se produce un error. */
  void insertarOrdenadoPorSimbolo(TLista &l, TElemento elem, TError &error)
  {
       TLista nuevo;
         
       if ( (listaVacia(l)) || (elem.simbolo < l->elem.simbolo) )
         {   
                error = NoError;
                nuevo = new (TNodo);
                nuevo->elem = elem;
                nuevo->sig = l;
                l = nuevo;
             
         }
         else if (elem.simbolo==l->elem.simbolo)
         {  
            error = ErrorElementoYaExiste;
         }
         else
         {   
            insertarOrdenadoPorSimbolo(l->sig,elem,error);
         } 
  }
}
